package com.mortals.xhx.module.matter.web;

import cn.hutool.setting.Setting;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.framework.model.Result;
import com.mortals.framework.util.DataUtil;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.framework.config.InterceptorConfig;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.JointInfoShowEnum;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.service.MatterService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

import static com.mortals.framework.ap.SysConstains.MESSAGE_INFO;
import static com.mortals.framework.ap.SysConstains.PAGEINFO_KEY;

/**
 * 基础事项
 *
 * @author zxfei
 * @date 2022-01-20
 */
@RestController
@RequestMapping("matter")
public class MatterController extends BaseCRUDJsonBodyMappingController<MatterService, MatterEntity, Long> {

    @Autowired
    private ParamService paramService;
    @Autowired
    private InterceptorConfig interceptorConfig;
//    @Autowired
//    private SiteThemeService siteThemeService;

    public MatterController() {
        super.setFormClass(MatterForm.class);
        super.setModuleDesc("基础事项");
    }

    /**
     * @param query
     * @param model
     * @param context
     * @throws AppException
     */
    @Override
    protected void doListBefore(MatterEntity query, Map<String, Object> model, Context context) throws AppException {
        if(ObjectUtils.isEmpty(query.getEventTypeShowNotList())){
            ArrayList<String> notList = new ArrayList<>();
            notList.add("行政处罚");
            query.setEventTypeShowNotList(notList);
        }

        super.doListBefore(query, model, context);
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "appoveObjectShow", paramService.getParamBySecondOrganize("Matter", "appoveObjectShow"));
        this.addDict(model, "operatScopeShow", paramService.getParamBySecondOrganize("Matter", "operatScopeShow"));
        this.addDict(model, "appoveTimeLimitShow", paramService.getParamBySecondOrganize("Matter", "appoveTimeLimitShow"));
        this.addDict(model, "handleType", paramService.getParamBySecondOrganize("Matter", "handleType"));
        this.addDict(model, "isChargesShow", paramService.getParamBySecondOrganize("Matter", "isChargesShow"));
        this.addDict(model, "certificationLevelsShow", paramService.getParamBySecondOrganize("Matter", "certificationLevelsShow"));
        this.addDict(model, "windowToTheSceneNum", paramService.getParamBySecondOrganize("Matter", "windowToTheSceneNum"));
        this.addDict(model, "isOnlineSubscribeShow", paramService.getParamBySecondOrganize("Matter", "isOnlineSubscribeShow"));
        this.addDict(model, "isExpressTakeShow", paramService.getParamBySecondOrganize("Matter", "isExpressTakeShow"));
        this.addDict(model, "isProvinceAcquisitionShow", paramService.getParamBySecondOrganize("Matter", "isProvinceAcquisitionShow"));
        this.addDict(model, "isApplyProvinceShow", paramService.getParamBySecondOrganize("Matter", "isApplyProvinceShow"));
        this.addDict(model, "onlineType", paramService.getParamBySecondOrganize("Matter", "onlineType"));
        this.addDict(model, "onlineToTheSceneNum", paramService.getParamBySecondOrganize("Matter", "onlineToTheSceneNum"));
        this.addDict(model, "onlineOperatDeep", paramService.getParamBySecondOrganize("Matter", "onlineOperatDeep"));
        this.addDict(model, "isExpressTakeOnlineShow", paramService.getParamBySecondOrganize("Matter", "isExpressTakeOnlineShow"));
        this.addDict(model, "isDoorTakeShow", paramService.getParamBySecondOrganize("Matter", "isDoorTakeShow"));
        this.addDict(model, "eventTypeShow", paramService.getParamBySecondOrganize("Matter", "eventTypeShow"));
        this.addDict(model, "performHierarchyShow", paramService.getParamBySecondOrganize("Matter", "performHierarchyShow"));
        this.addDict(model, "powerSourceShow", paramService.getParamBySecondOrganize("Matter", "powerSourceShow"));
        this.addDict(model, "performDeptTypeShow", paramService.getParamBySecondOrganize("Matter", "performDeptTypeShow"));
        this.addDict(model, "goveServiceCenterShow", paramService.getParamBySecondOrganize("Matter", "goveServiceCenterShow"));
        this.addDict(model, "isConvenientCenterShow", paramService.getParamBySecondOrganize("Matter", "isConvenientCenterShow"));
        this.addDict(model, "terminalHandle", paramService.getParamBySecondOrganize("Matter", "terminalHandle"));
        this.addDict(model, "isOnline", paramService.getParamBySecondOrganize("Matter", "isOnline"));
        this.addDict(model, "isOnlinePayShow", paramService.getParamBySecondOrganize("Matter", "isOnlinePayShow"));
        this.addDict(model, "entrustmentDepartmen", paramService.getParamBySecondOrganize("Matter", "entrustmentDepartmen"));
        this.addDict(model, "matterStatus", paramService.getParamBySecondOrganize("Matter", "matterStatus"));
        this.addDict(model, "type", paramService.getParamBySecondOrganize("Matter", "type"));
        this.addDict(model, "mattertype", paramService.getParamBySecondOrganize("Matter", "mattertype"));
        this.addDict(model, "eventType", paramService.getParamBySecondOrganize("Matter", "eventType"));
        this.addDict(model, "parson", paramService.getParamBySecondOrganize("Matter", "parson"));
        this.addDict(model, "lengal", paramService.getParamBySecondOrganize("Matter", "lengal"));
        this.addDict(model, "source", paramService.getParamBySecondOrganize("Matter", "source"));
        this.addDict(model, "jointInfoShow", JointInfoShowEnum.getEnumMap());

        Setting baseInfoSetting = interceptorConfig.getBaseInfoSetting();
        Map<String, String> baseInfoMap = new HashMap<>();
        baseInfoSetting.entrySet().stream().forEach(item -> {
            if (!ObjectUtils.isEmpty(item.getValue())) {
                baseInfoMap.put(item.getValue(), item.getKey());
            }
        });
        this.addDict(model, "baseInfo", baseInfoMap);

        Setting sqclInfoSetting = interceptorConfig.getSqclInfoSetting();
        Map<String, String> sqclInfoMap = new HashMap<>();
        sqclInfoSetting.entrySet().stream().forEach(item -> {
            if (!ObjectUtils.isEmpty(item.getValue())) {
                sqclInfoMap.put(item.getValue(), item.getKey());
            }
        });
        this.addDict(model, "sqclInfo", sqclInfoMap);
        //theme
//        Map<String, String> themeMap = siteThemeService.find(new SiteThemeQuery()).stream().collect(Collectors.toMap(x -> x.getThemeCode(), y -> y.getThemeName(), (o, n) -> n));
//        this.addDict(model, "theme", themeMap);
        super.init(model, context);
    }


    /**
     * 添加事项到站点
     */
    @PostMapping(value = "addMatterToSite")
    public String addBusinessToSite(@RequestBody Map<String, Object> map) {
        JSONObject jsonObject = new JSONObject();
        Map<String, Object> model = new HashMap<String, Object>();
        String matterIds = (String) map.get("matterIds");
        Long siteId = DataUtil.converStr2Long(map.get("siteId").toString(), 0L);

        try {
            Rest<String> rest = this.service.addMatterToSite(matterIds, siteId, getContext());
            jsonObject.put(KEY_RESULT_MSG, rest.getMsg());
            jsonObject.put(KEY_RESULT_DATA, model);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            this.recordSysLog(this.request, rest.getMsg());
        } catch (Exception e) {
            log.error("获取异常", e);
            jsonObject.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            jsonObject.put(KEY_RESULT_MSG, super.convertException(e));
        }
        return jsonObject.toJSONString();
    }


    @PostMapping(value = "sublist")
    @UnAuth
    public Rest<Object> sublist(@RequestBody MatterEntity query) {
        Rest<Object> ret = new Rest<>();
        Map<String, Object> model = new HashMap<>();
        Context context = this.getContext();
        String busiDesc = "查询差集" + this.getModuleDesc();
        int code=VALUE_RESULT_SUCCESS;
        try {
            PageInfo pageInfo = buildPageInfo(query);
            Result<MatterEntity> result = this.getService().findSubList(query, pageInfo, context);
            model.put(KEY_RESULT_DATA, result.getList());
            model.put(PAGEINFO_KEY, result.getPageInfo());
            parsePageInfo(model, result.getPageInfo());
            model.put(MESSAGE_INFO, busiDesc + "成功");
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        this.init(model, context);
        ret.setCode(code);
        ret.setData(model);
        ret.setMsg(model.get(MESSAGE_INFO) == null ? "" : model.remove(MESSAGE_INFO).toString());
        return ret;
    }

}