package com.mortals.xhx.module.site.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.module.matter.model.MatterEntity;
import com.mortals.xhx.module.matter.model.MatterQuery;
import com.mortals.xhx.module.matter.service.MatterService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.site.dao.SiteMatterDao;
import com.mortals.xhx.module.site.model.SiteMatterEntity;
import com.mortals.xhx.module.site.service.SiteMatterService;
import org.springframework.util.ObjectUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * SiteMatterService
 * 站点事项 service实现
 *
 * @author zxfei
 * @date 2022-01-12
 */
@Service("siteMatterService")
public class SiteMatterServiceImpl extends AbstractCRUDServiceImpl<SiteMatterDao, SiteMatterEntity, Long> implements SiteMatterService {

    @Autowired
    private MatterService matterService;

    @Override
    protected void findAfter(SiteMatterEntity params, PageInfo pageInfo, Context context, List<SiteMatterEntity> list) throws AppException {
      //  List<Long> matterIdlist = list.parallelStream().map(item -> item.getMatterId()).collect(Collectors.toList());
        //Map<Long, MatterEntity> matterEntityMap = matterService.find(new MatterQuery().idList(matterIdlist)).parallelStream().collect(Collectors.toMap(x -> x.getId(), y -> y, (o, n) -> n));
        list.forEach(item -> {
            if (!ObjectUtils.isEmpty(item.getMatterId())) {
                MatterEntity matterEntity = matterService.get(item.getMatterId());
                if (!ObjectUtils.isEmpty(matterEntity)) {
                    item.setBelongDept(matterEntity.getBelongDept());
                    item.setWindowToTheSceneNum(matterEntity.getWindowToTheSceneNum());
                    item.setOnlineToTheSceneNum(matterEntity.getOnlineToTheSceneNum());
                    item.setDeptCode(matterEntity.getDeptCode());
                    item.setAreaCode(matterEntity.getAreaCode());
                }
            }
        });
        super.findAfter(params, pageInfo, context, list);
    }

    /**
     * @param entity
     * @param context
     * @throws AppException
     */
    @Override
    protected void updateBefore(SiteMatterEntity entity, Context context) throws AppException {
        super.updateBefore(entity, context);
        SiteMatterEntity beforeSiteMatterEntity = this.get(entity.getId(), context);
        if (!ObjectUtils.isEmpty(beforeSiteMatterEntity.getDeptId()) && !beforeSiteMatterEntity.getDeptId().equals(entity.getDeptId())) {
            //更新事项中的部门编号
            MatterEntity matterEntity = matterService.get(entity.getMatterId(), context);
            if (!ObjectUtils.isEmpty(matterEntity)) {
                matterEntity.setDeptCode(entity.getDeptCode());
                matterEntity.setDeptName(entity.getDeptName());
                matterService.update(matterEntity, context);
            }
        }
    }

    @Override
    public void deleteBysiteIdAndSource(Long siteId, Integer source, Context context) {
        Map<String, Object> condition = new HashMap<>();
        condition.put("siteId", siteId);
        condition.put("source", source);
        this.dao.delete(condition);
    }
}